/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2018 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name    : r_cg_serial_user.c
* Version      : CodeGenerator for RL78/G13 V2.05.03.01 [12 Nov 2018]
* Device(s)    : R5F100LG
* Tool-Chain   : CCRL
* Description  : This file implements device driver for Serial module.
* Creation Date: 
***********************************************************************************************************************/

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_serial.h"
/* Start user code for include. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
#include "r_cg_userdefine.h"

/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
#pragma interrupt r_csi11_interrupt(vect=INTCSI11)
/* Start user code for pragma. Do not edit comment generated here */
//R_PRAGMA_FREERTOS_INTERRUPT(r_uart0_interrupt_receive)
//#define r_uart0_interrupt_receive _r_uart0_interrupt_receive
//R_PRAGMA_FREERTOS_INTERRUPT(r_uart0_interrupt_error)
//#define r_uart0_interrupt_error _r_uart0_interrupt_error
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
extern volatile uint8_t * gp_csi11_rx_address;         /* csi11 receive buffer address */
extern volatile uint16_t  g_csi11_rx_length;           /* csi11 receive data length */
extern volatile uint16_t  g_csi11_rx_count;            /* csi11 receive data count */
extern volatile uint8_t * gp_csi11_tx_address;         /* csi11 send buffer address */
extern volatile uint16_t  g_csi11_send_length;         /* csi11 send data length */
extern volatile uint16_t  g_csi11_tx_count;            /* csi11 send data count */
/* Start user code for global. Do not edit comment generated here */
extern volatile uint8_t * gp_uart0_err_type;           /* uart0 reception error type */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: r_csi11_interrupt
* Description  : This function is INTCSI11 interrupt service routine.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void __near r_csi11_interrupt(void)
{
    volatile uint8_t err_type;

    err_type = (uint8_t)(SSR03 & _0001_SAU_OVERRUN_ERROR);
    SIR03 = (uint16_t)err_type;

    if (1U == err_type)
    {
        r_csi11_callback_error(err_type);    /* overrun error occurs */
    }
    else
    {
        if (g_csi11_tx_count > 0U) 
        {
            if (g_csi11_tx_count != (g_csi11_send_length - 1U)) 
            {
                *gp_csi11_rx_address = SIO11;
                gp_csi11_rx_address++;
            }
            
            SIO11 = *gp_csi11_tx_address;
            gp_csi11_tx_address++;
            g_csi11_tx_count--;
        }
        else 
        {
            if ((SMR03 & _0001_SAU_BUFFER_EMPTY) == 1U) 
            {
                r_csi11_callback_sendend();    /* complete send */
                *gp_csi11_rx_address = SIO11;
                gp_csi11_rx_address++;    
                
                if ((SSR03 & _0040_SAU_UNDER_EXECUTE) == 0U)
                {
                    *gp_csi11_rx_address = SIO11;
                    r_csi11_callback_receiveend();    /* complete receive */
                }
                else
                {
                    SMR03 &= ~_0001_SAU_BUFFER_EMPTY;
                }
            }
            else 
            {
                *gp_csi11_rx_address = SIO11;
                r_csi11_callback_receiveend();    /* complete receive */
            }
        }
    }
}

/***********************************************************************************************************************
* Function Name: r_csi11_callback_receiveend
* Description  : This function is a callback function when CSI11 finishes reception.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void r_csi11_callback_receiveend(void)
{
    /* Start user code. Do not edit comment generated here */
    /* End user code. Do not edit comment generated here */
}

/***********************************************************************************************************************
* Function Name: r_csi11_callback_error
* Description  : This function is a callback function when CSI11 reception error occurs.
* Arguments    : err_type -
*                    error type value
* Return Value : None
***********************************************************************************************************************/
static void r_csi11_callback_error(uint8_t err_type)
{
    /* Start user code. Do not edit comment generated here */
    /* End user code. Do not edit comment generated here */
}

/***********************************************************************************************************************
* Function Name: r_csi11_callback_sendend
* Description  : This function is a callback function when CSI11 finishes transmission.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void r_csi11_callback_sendend(void)
{
    /* Start user code. Do not edit comment generated here */
    /* End user code. Do not edit comment generated here */
}

/* Start user code for adding. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
